@@ -2,7 +2,6 @@
 
 import ast
 import dis
-import re
 import sys
 import urllib.request
 from functools import lru_cache
@@ -10,6 +9,7 @@ from pathlib import Path
 
 from hypothesis import assume, strategies as st
 from hypothesis.extra.lark import LarkStrategy
+from hypothesis.internal.charmap import _union_intervals
 from lark import Lark
 from lark.indenter import Indenter
 
@@ -35,19 +35,25 @@ COMPILE_MODES = {
 }
 
 
+def _chars_to_regex_set(s: list) -> str:
+    spans = tuple((ord(c), ord(c)) for c in s)
+    return "".join(
+        chr(a) if a == b else f"{chr(a)}-{chr(b)}"
+        for a, b in _union_intervals(spans, spans)
+    )
+
+
 @lru_cache()
 def identifiers() -> st.SearchStrategy[str]:
-    _lead = []
-    _subs = []
+    lead = []
+    subs = []
     for c in map(chr, range(sys.maxunicode + 1)):
-        if not utf8_encodable(c):
-            continue
-        if c.isidentifier():
-            _lead.append(c)  # e.g. "a"
-        if ("_" + c).isidentifier():
-            _subs.append(c)  # e.g. "1"
-    pattern = "[{}][{}]*".format(re.escape("".join(_lead)), re.escape("".join(_subs)))
-    return st.from_regex(pattern, fullmatch=True).filter(str.isidentifier)
+        if utf8_encodable(c) and ("_" + c).isidentifier():
+            subs.append(c)  # e.g. "1"
+            if c.isidentifier():
+                lead.append(c)  # e.g. "a"
+    pattern = f"[{_chars_to_regex_set(lead)}][{_chars_to_regex_set(subs)}]*"
+    return st.from_regex(pattern, fullmatch=True)
 
 
 class PythonIndenter(Indenter):
@@ -121,9 +127,15 @@ class GrammarStrategy(LarkStrategy):
                 # of bizzare errors in CPython, especially with the new PEG parser,
                 # and so we maintain this extra clause to ensure that we get a decent
                 # error message out of it.
-                if isinstance(err, SystemError) and sys.version_info[:3] == (3, 9, 0):
+                if isinstance(err, SystemError) and (
+                    sys.version_info[:3] == (3, 9, 0)
+                    or sys.version_info[:3] >= (3, 9, 8)
+                    and str(err) == "Negative size passed to PyUnicode_New"
+                ):
                     # We've triggered https://bugs.python.org/issue42218 - it's been
                     # fixed upstream, so we'll treat it as if it were a SyntaxError.
+                    # Or the new https://bugs.python.org/issue45738 which makes me
+                    # wish CPython would start running proptests in CI already.
                     assume(False)
                 source_code = ascii("".join(draw_state.result[count:]))
                 raise type(err)(
